#!/etc/Tivoli/bin/perl
#
# The select_gateway_policy determines the set of gateways that are
# allowed to manage the endpoint. select_gateway_policy is run at
# initial login, when an endpoint is isolated, or when an endpoint
# performs a migratory login.
#
# If the script does not return gateways, the endpoint manager performs
# its default selection process and generates a list of up to five gateways
# in the endpoints region. The endpoint managers default selection
# process is overridden by the select_gateway_policy. If this policy is
# not defined, the intercepting gateway is added to the end of the list of
# candidate gateways.
#
# The intercepting gateway is also added to the end of the
# select_gateway_policy list to ensure that the endpoint has at least one
# definite contact. If the endpoint manager cannot contact any of the
# gateways listed in the script, the endpoint manager assigns the
# intercepting gateway to the endpoint. If the script fails (returns
# non-zero), the login attempt fails. If the select_gateway_policy runs
# successfully, the object ID of the assigned gateway is passed to the
# endpoint.
#
# The endpoint manager runs select_gateway_policy when it receives
# the endpoints login packet. The following command line arguments are 
# passed to this script from the Endpoint Manager.
#
#	$1 - The label of the endpoint machine
#	$2 - The object reference of the endpoint machine
#	$3 - The architecture type of the endpoint machine
#	$4 - The object reference of the gateway that the endpoint logged into
#	$5 - The ip address of the endpoint logging in.
#	$6 - region
#	$7 - dispatcher
#	$8 - version
#
# This example select gateway policy uses an external configuration
# file (epmgr.cfg) to determine which gateway the endpoint should be
# mapped to.  The epmgr.cfg file contains IP subnet patterns paired
# with a corresponding gateway name.  Using the IP address of the 
# endpoint, this script attempts to match it with the pattern IP's in
# the configuration file.  For each match, this script outputs the
# gateway OID, one per line.  These values are used by the endpoint
# manager to connect the endpoint to the suggested gateways.
#
# Note, the configuration file referenced in this script must be
# present on the endpoint manager (TMR Server).  Ideally, it should
# be the same file used by the example login_policy script as well.
# See the login_policy script for more information about this file.



################################################
# CONFIGURATION PARAMETERS 
################################################

$CONFIGFILE="c:/temp/epmgr.cfg";

#$DEBUG = 1;
$DEBUGFILE="c:/temp/selectgw.dbg";
open(DBG, ">>$DEBUGFILE") if($DEBUG);


################################################
# Main 
################################################
if($#ARGV != 7)
{
	print STDERR "Usage: $0 <epname> <epoid> <eparch> <gatewayoid> <epipaddr> <epregion> <epdispatch> <epversion>\n";
	exit 1;
}

($epname,$epoid,$eparch,$gatewayoid,$epipaddr,$epregion,$epdispatch,$epversion) = @ARGV;

@MappedGateways = &findMappedGateways($epipaddr);

foreach $Gateway (@MappedGateways)
{
	$GatewayOid = `wlookup -o -r Gateway $Gateway 2>/dev/null`;
	if($GatewayOid)
	{
		chop $GatewayOid;
		print DBG "$GatewayOid\n" if($DEBUG);
		print "$GatewayOid\n";
	}
	else
	{
		print DBG "Could not lookup Gateway $Gateway\n" if($DEBUG);
	}
}
exit 0;
	



###############################################################################
# This subroutine reads the configuration file and attempts to match the
# given IP address with the pattern in the file.  If any match, return
# the gateway name of each one.
#
# Here are some examples of possible IP patterns and corresponding gateways
#
# 123.123.123.*         mn1-gateway     # Wildcard last octet
# 123.123.[1-100].*     mn2-gateway     # Example showing ranges
# 123.123.[101-123].*   mn3-gateway     # Example showing ranges
# 123.123.*.*           mn4-gateway     # More wildcard examples
# *.*.*.*               mn5-gateway     # Default if none of the above match#
#
sub findMappedGateways
{
	local($EpIp) = @_;
	local(@EpIpSplit) = split(/[\.+]/, $EpIp);
	local(@Gateways);

	open(CONFIGFILE, "<$CONFIGFILE") || die "Unable to read configuration file $CONFIGFILE\n";
	while(<CONFIGFILE>)
	{
		s/#.*$//;
		if(/\s*([^\.\s]*)\.([^\.\s]*)\.([^\.\s]*)\.([^\.\s]*)\s*([^\s]*)/)
		{
			if(&matchOctet($1, $EpIpSplit[0]) &&
			   &matchOctet($2, $EpIpSplit[1]) &&
			   &matchOctet($3, $EpIpSplit[2]) &&
			   &matchOctet($4, $EpIpSplit[3]))
			{
				print DBG "Matched $5 in config file\n" if ($DEBUG);
				push(@Gateways, $5);
			}
		}
	}
	close CONFIGFILE;
	return @Gateways;
}


sub matchOctet
{
	local($pattern, $octet) = @_;

	$pattern eq $octet && return 1;
	$pattern eq "*" && return 1;
	if($pattern =~ /\[(\d*)-(\d*)\]/)
	{
		$1 <= $octet && $octet <= $2 && return 1;
	}
	return 0;
}

